/* geomviewer.h: main S2PLOT code
 *
 * ==================================================================
 * S2PLOT is freely available for non-commercial and educational use,
 * however, it is not public-domain software. The programming library
 * and documentation may not be redistributed or sub-licensed in any
 * form without permission from Swinburne University of
 * Technology. The programming library and sample codes are provided
 * without warranty.
 *
 * We would appreciate it if research outcomes using S2PLOT would
 * provide the following acknowledgement:
 *
 * "Three-dimensional visualisation was conducted with the S2PLOT
 * progamming library"
 *
 * and a reference to 
 *
 * D.G.Barnes, C.J.Fluke, P.D.Bourke & O.T.Parry, 2006, Publications
 * of the Astronomical Society of Australia, 23(2), 82-93.
 *
 * S2PLOT development was funded through the Researcher Developer
 * Grant Scheme at Swinburne University of Technology.
 * ==================================================================
 *
 * $Id: geomviewer.h 5544 2011-05-23 23:43:25Z dbarnes $
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

// this MUST be before the s2glut.h include as it sets a define 
// that is examined when glext.h is implicitly included by gl.h.
#if defined(S2LINUX)
#define GL_GLEXT_PROTOTYPES
#endif

//#include "s2glut.h"
#include "s2opengl.h"
#include "s2win.h"

#include <sys/time.h>
#include <sys/resource.h>
#include <sys/file.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/times.h>

//#ifdef __APPLE__
//#include "hiddenMouseCursor.h"
//#endif

#include "s2base.h"
#include "s2const.h"
#include "paulslib.h"
#include "bitmaplib.h"
#include "opengllib.h"



// RO access to pmin, pmax for device drivers
XYZ _s2priv_pmin(void);
XYZ _s2priv_pmax(void);

// internal error reporting
void _s2debug(char *fn, char *messg, ...);
void _s2warn(char *fn, char *messg, ...);
void _s2error(char *fn, char *messg, ...);
void _s2warnk(char key, char *messg, ...);

/* Stereo.c */
void HandleDisplay(void);

void CreateOpenGL(void);
void MakeGeometry(int, int);
void MakeLighting(void);
void MakeMaterial(void);
void HandleReshape(int,int);
void HandleKeyboard(unsigned char key,int x, int y);
void HandleSpecialKeyboard(int key,int x, int y);
void RotateCamera(double,double,double,int);
void TranslateCamera(double,double,int);
void HandleMouse(int,int,int,int);
void HandleMouseMotion(int,int);
void HandlePassiveMotion(int,int);

void HandleSpaceballButton(int, int);
void HandleSpaceballMotion(int, int, int);
void HandleSpaceballRotate(int, int, int);


void HandleVisibility(int vis);
void HandleIdle(void);
int  ReadGeometry(char *);
void CleanGeometry(void);
#if defined(BUILDING_S2PLOT)
void SaveGeomFile(char *);
#else
void SaveGeomFile(void);
#endif
int  ReadOFF(char *);
void DeleteGeometry(void);
//-void CameraHome(int);
int  AutoPilot(int,char *);


void ReadViewFile(char *);
#if !defined(BUILDING_VIEWER)
void SaveViewFile(CAMERA, char *);
#else
void SaveViewFile(CAMERA);
#endif
void AppendCameraPosition(void);
void CreateAxes(void);
void DrawExtras(void);
void CreateProjection(int);
void AddLine2Database(XYZ,XYZ,COLOUR,COLOUR,double);
void AddFace2Database(XYZ *,int,COLOUR,double,XYZ);
void AddMarker2Database(int,double,XYZ,COLOUR);
//-void CalcBounds(void);
//-void UpdateBounds(XYZ);
void CleanExit(void);

#if (0)
// menus.c
void HandleMainMenu(int);
void HandleAnimMenu(int);
void HandleWindowMenu(int);
void HandleRenderMenu(int);
void HandleSpeedMenu(int);
void HandleCameraMenu(int);
void HandleStereoMenu(int);
void HandleAppearMenu(int);
void HandleFileMenu(int);
void HandleViewMenu(int);
void HandleProjectionMenu(int);

// dome.c
void DomeTextures(void);
int  DomeCreate(void);
void DrawDome(int,int);
void DomeSplitFace(DOMEFACE *,int *);
void DomeFlatten(XYZ *,double);
int  ReadMesh(char *);
void MakeDefaultMesh(void);
void CalcWarpData(void);
void EstimateWarp(double,double,double *,double *,double *);
void SetVertex(DOMEFACE);
int MirrorPosition(XYZ,double,XYZ,XYZ,XYZ *);
#endif

// misc.c
void CreateASphere(XYZ,double,int,int,int);
void CreateAPlanet(XYZ,double,int,int,int,float,XYZ,float);
void GiveUsage(char *);
int  ReadVector(FILE *,XYZ *);
int  ReadColour(FILE *,COLOUR *);


